--- ========================================================= ---
---  Setup Revision, Version, Date and add to AceLibrary
--- ========================================================= ---
local VERSION = tonumber(("$Revision: 43237 $"):match("%d+"))

local Sphere = Sphere
Sphere_SubButtons = Sphere:NewModule("SubButtons", "AceEvent-2.0", "AceHook-2.1")
local self = Sphere_SubButtons
if Sphere.revision < VERSION then
	Sphere.version = "r" .. VERSION
	Sphere.revision = VERSION
	Sphere.date = ("$Date: 2007-07-08 17:49:50 -0400 (Sun, 08 Jul 2007) $"):match("%d%d%d%d%-%d%d%-%d%d")
end
Sphere.desc = "Adds Buttons to the Sphere"

--- ========================================================= ---
---  Setup Locals
--- ========================================================= ---
local DogTag = AceLibrary("DogTag-1.0")
local G = AceLibrary('Gratuity-2.0')
local Deformat = AceLibrary("Deformat-2.0")

local L = Sphere:L("Sphere_SubButtons")

local data = {
	buttonCount = 0,
	menuCount = 0,
	subCount = 0,
	cursorType = nil,
	cursorInfo1 = nil,
	cursorInfo2 = nil,
	cursorHasItem = false,
}

--- ========================================================= ---
---  On Intitalize / Setup Database
--- ========================================================= ---
function Sphere_SubButtons:OnInitialize()
	self.db = Sphere:AcquireDBNamespace("SubButtons")
	Sphere:RegisterDefaults("SubButtons", "char", {
		settings = {
			scale = 0.90,
			offSet = 7,
		},
		SphereSubBar1 = {
			buttons = {},
		},
		SphereSubBar2 = {
			buttons = {},
		},
		SphereSubBar3 = {
			buttons = {},
		},
		SphereSubBar4 = {
			buttons = {},
		},
		SphereSubBar5 = {
			buttons = {},
		},
		SphereSubBar6 = {
			buttons = {},
		},
		SphereSubBar7 = {
			buttons = {},
		},
		SphereSubBar8 = {
			buttons = {},
		},
		SphereSubBar9 = {
			buttons = {},
		},
		SphereSubBar10 = {
			buttons = {},
		},
		
	})

	Sphere_SubButtons:CreateBars()
end

--- ========================================================= ---
---  Setup Options
--- ========================================================= ---
Sphere_SubButtons:RegisterSphereOption({
	name = L["Sub Buttons"],
	desc = L["Change the Sub Buttons options."],
	type = 'group',
	args = {
		appearance = {
			name = L["Appearance"],
			desc = L["Change how the Sub Buttons Appearance looks in general."],
			type = 'group',
			args = {
				size = {
					name = L["Size"],
					desc = L["Set how large the Sub Buttons are."],
					type = 'range',
					min = 0.50,
					max = 2,
					step = 0.01,
					bigStep = 0.05,
					get = function()
						return self.db.char.settings.scale
					end,
					set = "SetScale"
				},
				offSet = {
					name = L["offSet"],
					desc = L["Set how much of an offSet the Sub Buttons are."],
					type = 'range',
					min = 0,
					max = 10,
					step = 1,
					bigStep = 2,
					get = function()
						return self.db.char.settings.offSet
					end,
					set = "SetOffSet"
				},
			},
		},
	}

})

--- ========================================================= ---
---  On Enable
--- ========================================================= ---
function Sphere_SubButtons:OnEnable()
	self:ScheduleRepeatingEvent(self.UpdateButton, TOOLTIP_UPDATE_TIME, self)
	self:ScheduleRepeatingEvent(self.FlashButton, ATTACK_BUTTON_FLASH_TIME, self)
	
	self:ScheduleEvent("Sphere_SubButtons_PostEnable", self.OnPostEnable, 1, self)
end

--- ========================================================= ---
---  On Disable
--- ========================================================= ---
function Sphere_SubButtons:OnDisable()
	self:UnregisterAllEvents();

end

--- ========================================================= ---
---  On Post Enable
--- ========================================================= ---
function Sphere_SubButtons:OnPostEnable()
	for indexMenu=1, data.menuCount do
		for indexSub=1, data.subCount do
			Sphere_SubButtons:SetIcon(getglobal("SphereButton"..indexMenu.."Sub"..indexSub))
		end
	end
end

--- ========================================================= ---
--- Find Max Spell ID
--- ========================================================= ---
function Sphere_SubButtons:FindMaxID()
	local id = 10
	while GetSpellTexture(id, BOOKTYPE_SPELL) do
		id = id + 1
	end
	return id
end

--- ========================================================= ---
---  Set Scale
--- ========================================================= ---
function Sphere_SubButtons:SetScale(value)
	if value then
		self.db.char.settings.scale = value
	else
		value = self.db.char.settings.scale
	end
	
	for indexMenu=1, data.menuCount do
		for indexSub=1, data.subCount do
			getglobal("SphereButton"..indexMenu.."Sub"..indexSub):SetScale(value)
		end
	end
end

--- ========================================================= ---
---  Set offSet
--- ========================================================= ---
function Sphere_SubButtons:SetOffSet(value)
	if value then
		self.db.char.settings.offSet = value
	else
		value = self.db.char.settings.offSet
	end
	
	local index = 0
	local xDir, xLoc, yLoc
	local offsetY = {30, 6, 0, 0, -6, -30, -6, 0, 0, 6}
	local offSet = self.db.char.settings.offSet
	
	for indexMenu = 1, 10 do 
		local anchor = "SphereBar" .. indexMenu
		xDir = 1;
		if (indexMenu > 6) then
			xDir = -1;
		end
		for xLoc = 0, (320 - 32), 32 do 
			if (index >= 10) then
				index = 1
			else
				index = index + 1;
			end
			local name = "SphereButton"..indexMenu.."Sub"..index
			if (xDir == 1) then
				if (xLoc == 0) then
					getglobal(name):SetPoint("LEFT", getglobal(anchor), "RIGHT", offSet, offsetY[indexMenu]);
				else
					getglobal(name):SetPoint("LEFT", getglobal("SphereButton" .. tostring(indexMenu) .. "Sub" .. tostring(index - 1)), "RIGHT", offSet, 0);
				end
			else
				if (xLoc == 0) then
					getglobal(name):SetPoint("RIGHT", getglobal(anchor), "LEFT", -offSet, offsetY[indexMenu]);
				else
					getglobal(name):SetPoint("RIGHT", getglobal("SphereButton" .. tostring(indexMenu) .. "Sub" .. tostring(index - 1)), "LEFT", -offSet, 0);
				end
			end
		end
	end
end

--- ========================================================= ---
---  Create (10) Bars
--- ========================================================= ---
function Sphere_SubButtons:CreateBars()

	local index = 0

	for indexMenu=1, 10 do
		
		data.menuCount = data.menuCount + 1
		
		local name = "SphereSubBar"..indexMenu
		local anchor = "SphereBar" .. indexMenu
		
		local bar = CreateFrame("Frame", name, getglobal(anchor), "SecureStateHeaderTemplate")
	
		valsMenu = Sphere.db.namespaces.Buttons.char["SphereBar"..indexMenu]
		vals = Sphere.db.namespaces.SubButtons.char["SphereBar"..indexMenu]
		
		bar:SetID(indexMenu);
		bar:SetScale(self.db.char.settings.scale)
		bar:SetAlpha(1)
		
		if valsMenu.unit == "none" then
			bar:SetAttribute("unit", "")
		else
			bar:SetAttribute("unit", valsMenu.unit)
		end
		
		bar:SetAttribute("shown", valsMenu.shown)
		bar:SetAttribute("useunit", valsMenu.useunit)

		if valsMenu.shown == "unit" then
			RegisterUnitWatch(bar)
		end

		if not valsMenu.shown or valsMenu.shown == "always" or (valsMenu.shown == "nocombat" and not InCombatLockdown()) then
			bar:Show()
		else
			bar:Hide()
		end
		
		Sphere_SubButtons:UpdateBar(bar)
		
	end
	
	local xDir, xLoc, yLoc
	local offsetY = {32, 6, 0, 0, -6, -32, -6, 0, 0, 6}
	local offSet = self.db.char.settings.offSet
	
	for indexMenu = 1, 10 do 
		local anchor = "SphereBar" .. indexMenu
		xDir = 1;
		if (indexMenu > 6) then
			xDir = -1;
		end
		for xLoc = 0, (320 - 32), 32 do 
			if (index >= 10) then
				index = 1
			else
				index = index + 1;
			end
			local name = "SphereButton"..indexMenu.."Sub"..index
			if (xDir == 1) then
				if (xLoc == 0) then
					getglobal(name):SetPoint("LEFT", getglobal(anchor), "RIGHT", offSet, offsetY[indexMenu]);
				else
					getglobal(name):SetPoint("LEFT", getglobal("SphereButton" .. tostring(indexMenu) .. "Sub" .. tostring(index - 1)), "RIGHT", offSet, 0);
				end
			else
				if (xLoc == 0) then
					getglobal(name):SetPoint("RIGHT", getglobal(anchor), "LEFT", -offSet, offsetY[indexMenu]);
				else
					getglobal(name):SetPoint("RIGHT", getglobal("SphereButton" .. tostring(indexMenu) .. "Sub" .. tostring(index - 1)), "LEFT", -offSet, 0);
				end
			end
		end
	end
	
end

--- ========================================================= ---
--- Update Bar
--- ========================================================= ---
function Sphere_SubButtons:UpdateBar(bar)
	local state
	local statelist, driver = {}, {}, {}

	local valsMenu = Sphere.db.namespaces.Buttons.char[bar:GetParent():GetName()]
	local vals = Sphere.db.namespaces.SubButtons.char[bar:GetName()]

	if valsMenu.statemaps then
		for k, v in pairs(valsMenu.statemaps) do
			if string.find(k, "%-") then
				k = string.gsub(k, "%-", ":")
			end

			local add = true
			local list = v..":"..v
			table.insert(driver, "["..k.."] "..v)

			for _, l in ipairs(statelist) do
				if l == list then
					add = nil
					break
				end
			end

			if add then
				table.insert(statelist, v..":"..v)
			end
		end
	end

	driver = table.concat(driver, ";")
	state = SecureCmdOptionParse(driver)
	statelist = table.concat(statelist, ";")

	if #driver == 0 then
		state = "0"
	end

	RegisterStateDriver(bar, "state", driver)
	bar:SetAttribute("statemap-state", "$input")
	bar:SetAttribute("statebutton", statelist)
	bar:SetAttribute("state", state)
	bar:SetWidth(32)
	bar:SetHeight(32)

	for i = 1, 10 do 
		data.buttonCount = data.buttonCount + 1
		
		local btn = Sphere_SubButtons:CreateButton()
		btn:SetID(i)
		bar:SetAttribute("addchild", btn)
		
		vals.buttons[i] = vals.buttons[i] or {}
		for k, v in pairs(vals.buttons[i]) do
			btn:SetAttribute(k, v)
		end
		
		btn:SetAllPoints(bar);
		btn:SetFrameLevel(2)
	
	end
	
end

--- ========================================================= ---
--- Create Button
--- ========================================================= ---
function Sphere_SubButtons:CreateButton()
	
	if (data.subCount >= 10) then
		data.subCount = 1
	else
		data.subCount = data.subCount + 1;
	end
	
	local btn = CreateFrame("CheckButton", "SphereButton"..data.menuCount.."Sub"..data.subCount, UIParent, "SecureActionButtonTemplate, ActionButtonTemplate")
	btn:RegisterForClicks("AnyUp")
	btn:RegisterForDrag("LeftButton", "MiddleButton", "RightButton")
	btn:SetAttribute("checkselfcast", true)
	btn:SetAttribute("useparent-unit", true)
	btn:SetAttribute("useparent-statebutton", true)
	
	btn:SetScript("OnEnter", function(self) Sphere_SubButtons:ButtonEnter(self) end)
	btn:SetScript("OnLeave", function(self) Sphere_SubButtons:ButtonLeave(self) end)
	
	btn:SetScript("OnDragStart", function(self)
			if not InCombatLockdown() then
			
				local clickType = 1;
				if (arg1 == "RightButton") then
					clickType = 2;
				end
				if (arg1 == "MiddleButton") then
					clickType = 3;
				end
				Sphere_SubButtons:ButtonDragStart(self, clickType)
			end
		end)
	
	btn:SetScript("OnReceiveDrag", function() end)
	
	btn:SetScript("PreClick", function(self)
			if GetCursorInfo() and not InCombatLockdown() then
				self:SetScript("OnClick", nil)
				
				local clickType = 1;
				if (arg1 == "RightButton") then
					clickType = 2;
				end
				if (arg1 == "MiddleButton") then
					clickType = 3;
				end
				
				local type, info1, info2
				
				if GetCursorInfo() then
					type, info1, info2 = GetCursorInfo()
				end
				Sphere_SubButtons:ButtonReceiveDrag(self, clickType, type, info1, info2)
			end
		end)
		
	btn:SetScript("PostClick", function(self)
			self:SetChecked(nil)
			if not self:GetScript("OnClick") then
				self:GetParent():SetAttribute("addchild", self)
			end
		end)
	
	btn:SetScript("OnEvent", Sphere_Buttons.OnEvent);
	btn:RegisterEvent("PLAYER_ENTERING_WORLD");
	
	local cd = getglobal(btn:GetName().."Cooldown")
	cd:SetFrameLevel(cd:GetParent():GetFrameLevel() + 3)
	cd.text = cd:CreateFontString(nil, "OVERLAY")
	cd.text:SetPoint("CENTER")
	cd.text:SetFont("Fonts\\FRIZQT__.TTF", 22, "OUTLINE")
	
	local flash = getglobal(btn:GetName().."Flash")
	flash:ClearAllPoints()
	flash:SetAllPoints(cd)
	btn.flashtime = 0
	
	btn.style = "sub"
		
	btn:SetScript("OnAttributeChanged", function(self, name, value)
			Sphere_SubButtons:SaveAttrib(self, name, value)
		end)
		
	Sphere_Buttons:SkinButton(btn:GetName())
	
	return btn
end

--- ========================================================= ---
--- Button Enter
--- ========================================================= ---
function Sphere_SubButtons:ButtonEnter(this)
	if GetCursorInfo() then
		local type, info1, info2 = GetCursorInfo()
		data.cursorHasItem = true
		data.cursorType =  type
		data.cursorInfo1 =  info1
		data.cursorInfo2 =  info2
	else
		data.cursorType =  nil
		data.cursorInfo1 =  nil
		data.cursorInfo2 =  nil
	end
end

--- ========================================================= ---
--- Button Leave
--- ========================================================= ---
function Sphere_SubButtons:ButtonLeave(this)
	if not InCombatLockdown() and not Sphere.db.char.settings.locked then
		if (not(GetCursorInfo()) and (data.cursorHasItem)) then
			Sphere_SubButtons:ButtonReceiveDrag(this, 2, data.cursorType, data.cursorInfo1, data.cursorInfo2)
			data.cursorHasItem = false
		end
	end

	GameTooltip:Hide() 
end

--- ========================================================= ---
--- Button Drag Start
--- ========================================================= ---
function Sphere_SubButtons:ButtonDragStart(this, clickType)
	if not Sphere.db.char.settings.locked then
		local type, action = nil
	
		bar = "SphereBar"..this:GetParent():GetID()
		subbar = "SphereSubBar"..this:GetParent():GetID()

		if Sphere.db.namespaces.Buttons.char[bar].type == "stance" then
		
		elseif Sphere.db.namespaces.Buttons.char[bar].type == "modifiers" then
		
		elseif Sphere.db.namespaces.Buttons.char[bar].type == "harmful" then

		else
			if self.db.char[subbar].buttons[this:GetID()]["*type"..clickType] then
				type = self.db.char[subbar].buttons[this:GetID()]["*type"..clickType]
				self.db.char[subbar].buttons[this:GetID()]["*type"..clickType] = nil
				if clickType == 1 then
					self.db.char[subbar].buttons[this:GetID()]["*type*"] = nil
					this:SetAttribute("*type*", nil)
				end
			end
			
			if type then
				action = self.db.char[subbar].buttons[this:GetID()]["*"..type..""..clickType]
				self.db.char[subbar].buttons[this:GetID()]["*"..type..""..clickType] = nil
				if clickType == 1 then
					self.db.char[subbar].buttons[this:GetID()]["*"..type.."*"] = nil
					this:SetAttribute("*"..action.."*", nil)
				end
			end
			
			if type == "spell" then
				PickupSpell(action)
			elseif type == "item" then
				local itemLoc, itemA, itemB = Sphere_Buttons:FindItemLocation(action)
				if itemLoc == "Container" then
					PickupContainerItem(itemA, itemB)
				elseif itemLoc == "Inventory" then
					PickupInventoryItem(itemA)
				end
			elseif type == "macro" then
				PickupMacro(action)
			end
		end
	end
end

--- ========================================================= ---
--- Button Drag Stop
--- ========================================================= ---
function Sphere_SubButtons:ButtonReceiveDrag(this, clickType, type, info1, info2)
	if not Sphere.db.char.settings.locked then
		local state = this:GetAttribute("state-parent")
		if state == "0" then
			state = nil
		end
		
		if state then
			state = "-"..state
		else
			state = "*"
		end
		
		local unit = SecureButton_GetModifiedUnit(this, (state or "LeftButton"))
		local useunit = this:GetParent():GetAttribute("useunit")
		if useunit and unit and unit ~= "none" and UnitExists(unit) then
			if UnitCanAttack("player", unit) then
				this:SetAttribute("*harmbutton"..state, "nuke"..state)
				state = "-nuke"..state
			elseif UnitCanAssist("player", unit) then
				this:SetAttribute("*helpbutton"..state, "heal"..state)
				state = "-heal"..state
			end
		end
	
		if clickType == 1 then
			this:SetAttribute("*type*", type)
			if type == "spell" then
				local spell, rank = GetSpellName(info1, info2)
				this:SetAttribute("*spell"..state, spell)
				this:SetAttribute("*item"..state, nil)
				this:SetAttribute("*itemid"..state, nil)
				this:SetAttribute("*macro"..state, nil)
			elseif type == "item" then
				local item = GetItemInfo(info1)
				this:SetAttribute("*spell"..state, nil)
				this:SetAttribute("*item"..state, item)
				this:SetAttribute("*itemid"..state, info1)
				this:SetAttribute("*macro"..state, nil)
			elseif type == "macro" then
				this:SetAttribute("*spell"..state, nil)
				this:SetAttribute("*item"..state, nil)
				this:SetAttribute("*itemid"..state, nil)
				this:SetAttribute("*macro"..state, info1)
			end
		end
		
		local value = this:GetAttribute("value-parent")
		if value == "0" then
			value = nil
		end
		
		if value then
			value = "-"..value
		else
			value = clickType
		end
		
		local unit = SecureButton_GetModifiedUnit(this, (value or "LeftButton"))
		local useunit = this:GetParent():GetAttribute("useunit")
		if useunit and unit and unit ~= "none" and UnitExists(unit) then
			if UnitCanAttack("player", unit) then
				this:SetAttribute("*harmbutton"..value, "nuke"..value)
				value = "-nuke"..value
			elseif UnitCanAssist("player", unit) then
				this:SetAttribute("*helpbutton"..value, "heal"..value)
				value = "-heal"..value
			end
		end

		this:SetAttribute("*type"..value, type)
		if type == "spell" then
			local spell, rank = GetSpellName(info1, info2)
			this:SetAttribute("*spell"..value, spell)
			this:SetAttribute("*item"..value, nil)
			this:SetAttribute("*itemid"..value, nil)
			this:SetAttribute("*macro"..value, nil)
		elseif type == "item" then
			local item = GetItemInfo(info1)
			this:SetAttribute("*spell"..value, nil)
			this:SetAttribute("*item"..value, item)
			this:SetAttribute("*itemid"..value, info1)
			this:SetAttribute("*macro"..value, nil)
		elseif type == "macro" then
			this:SetAttribute("*spell"..value, nil)
			this:SetAttribute("*item"..value, nil)
			this:SetAttribute("*itemid"..value, nil)
			this:SetAttribute("*macro"..value, info1)
		end
		ClearCursor()
		
		data.cursorHasItem = false
		data.cursoractionType =  nil
		data.cursorInfo1 =  nil
		data.cursorInfo2 =  nil
	end
end

--- ========================================================= ---
--- Update Button
--- ========================================================= ---
function Sphere_SubButtons:UpdateButton()
	for indexMenu=1, data.menuCount do
		for indexSub=1, data.subCount do
			local btn = getglobal("SphereButton"..indexMenu.."Sub"..indexSub)
			local r, g, b, a = 1, 1, 1, 1
			local unit = SecureButton_GetUnit(btn)
			if btn.type == "spell" then
				local usable, nomana = IsUsableSpell(btn.spell) -- need check here
				if nomana then
					r, g, b, a = unpack(Sphere.db.namespaces.Buttons.char.colors.oom)
				elseif not usable then
					r, g, b, a = unpack(Sphere.db.namespaces.Buttons.char.colors.unusablecolor)
				elseif SpellHasRange(btn.spell) and IsSpellInRange(btn.spell, unit) == 0 then
					r, g, b, a = unpack(Sphere.db.namespaces.Buttons.char.colors.oor)
				end
			elseif btn.type == "item" then
				if not IsUsableItem(btn.spell) then
					r, g, b, a = unpack(Sphere.db.namespaces.Buttons.char.colors.unusable)
				elseif ItemHasRange(btn.spell) and IsItemInRange(btn.spell, unit) == 0 then
					r, g, b, a = unpack(Sphere.db.namespaces.Buttons.char.colors.oor)
				end
			elseif btn.type == "macro" and GetMacroBody(btn.spell) then
				local action = GetActionFromMacroText(GetMacroBody(btn.spell))
				if action then
					if GetItemInfo(action) then
						if not IsUsableItem(action) then
							r, g, b, a = unpack(Sphere.db.namespaces.Buttons.char.colors.unusable)
						elseif ItemHasRange(action) and IsItemInRange(action, unit) == 0 then
							r, g, b, a = unpack(Sphere.db.namespaces.Buttons.char.colors.oor)
						end
					else
						local usable, nomana = IsUsableSpell(action)
						if nomana then
							r, g, b, a = unpack(Sphere.db.namespaces.Buttons.char.colors.oom)
						elseif not usable then
							r, g, b, a = unpack(Sphere.db.namespaces.Buttons.char.colors.unusable)
						elseif SpellHasRange(action) and IsSpellInRange(action, unit) == 0 then
							r, g, b, a = unpack(Sphere.db.namespaces.Buttons.char.colors.oor)
						end
					end
				end
				Sphere_SubButtons:SetIcon(btn)
			end
			getglobal(btn:GetName().."Icon"):SetVertexColor(r, g, b, a)
		end
	end
end

--- ========================================================= ---
--- Set Icon
--- ========================================================= ---
function Sphere_SubButtons:SetIcon(btn)
	local value = btn:GetAttribute("state-parent")
	if value then
		value = Sphere_SubButtons:ConvertButton(btn, value)
		local texture, spell
		local text = getglobal(btn:GetName().."Name")
		local new = SecureButton_GetModifiedAttribute(btn, "type", value)
		btn.type = new
		btn.attack = nil
		text:SetText(nil)
		if new then
			spell = SecureButton_GetModifiedAttribute(btn, new, value)
			if spell then
				if new == "spell" then
					texture = GetSpellTexture(spell)
					if IsAttackSpell(spell) or IsAutoRepeatSpell(spell) then
						btn.attack = true
					end
				elseif new == "item" then
					local itemid = SecureButton_GetModifiedAttribute(btn, "itemid", value)
					if itemid then
						texture = select(10, GetItemInfo(itemid))
					end
				elseif new == "macro" then
					spell, texture = GetMacroInfo(spell)
				end
			end
			btn.spell = spell
		end
		
		local icon = getglobal(btn:GetName().."Icon")
		if icon:GetTexture() ~= texture then
			icon:SetTexture(texture)
			if texture then
				btn:SetAlpha(1)
				UIFrameFadeIn(icon, 0.50)
			end
			Sphere_SubButtons:UpdateItem(btn)
			Sphere_Buttons:UpdateCooldown(btn)
		end
			
		if not texture and not btn.grid then
			btn:SetAlpha(0)
		end
	end
end

--- ========================================================= ---
--- Update Item
--- ========================================================= ---
function Sphere_SubButtons:UpdateItem(btn)
	if btn.spell then
		local count = getglobal(btn:GetName().."Count")
		local border = getglobal(btn:GetName().."Border")

		if btn.type == "spell" then
			local spellid = Sphere_Buttons:GetSpellInfo(btn.spell)
			G:SetSpell(spellid, "spell")
			
			local reg
			local reagent = G:Find(SPELL_REAGENTS, 2, 99, false, true, false) and true
			
			if reagent then
				for i = 1, G:NumLines() do
					if (G:Find(SPELL_REAGENTS, i, i, false, true, false) and true) then
						reg = G:GetLine(i)
						reg= string.gsub(reg, SPELL_REAGENTS, "")
						if string.find(reg, "^|c") then
							_, _, reg = string.find(reg, "|c%x+(.*)|r")
						end
						reg = string.gsub(reg, "%(%)", "")
					end
				end
			end

			if reg then
				local num = GetItemCount(reg)
				if num > 1 then
					count:SetText(num)
				else
					count:SetText(nil)
				end
			else
				count:SetText(nil)
			end
			border:Hide()
			
		elseif btn.type == "item" then
			local num = GetItemCount(btn.spell)
			if num > 1 then
				count:SetText(num)
			else
				count:SetText(nil)
			end
		end
	end
end

--- ========================================================= ---
--- Convert Button
--- ========================================================= ---
function Sphere_SubButtons:ConvertButton(btn, value)
	local unit = SecureButton_GetAttribute(btn, "unit")
	if unit and unit ~= "none" and UnitExists(unit) then
		if UnitCanAttack("player", unit) then
			value = SecureButton_GetModifiedAttribute(btn, "harmbutton", value) or value
		elseif UnitCanAssist("player", unit) then
			value = SecureButton_GetModifiedAttribute(btn, "helpbutton", value) or value
		end
	end
	return value
end

--- ========================================================= ---
--- Save Attributes
--- ========================================================= ---
function Sphere_SubButtons:SaveAttrib(self, name, value)
	if string.find(name, "^%*") or string.find(name, "^shift") or string.find(name, "^ctrl") or string.find(name, "^alt") then
		Sphere.db.namespaces.SubButtons.char[self:GetParent():GetName()].buttons[self:GetID()][name] = value
		name = "state-parent"
	end

	if name == "state-parent" then
		Sphere_SubButtons:SetIcon(self)
	end
end

--- ========================================================= ---
--- Flash Button
--- ========================================================= ---
function Sphere_SubButtons:FlashButton()
	for indexMenu=1, data.menuCount do
		for indexSub=1, data.subCount do
			local btn = getglobal("SphereButton"..indexMenu.."Sub"..indexSub)
			local flash = getglobal(btn:GetName().."Flash")
			if btn.flash and btn.attack then
				if flash:IsVisible() then
					flash:Hide()
				else
					flash:Show()
				end
			else
				flash:Hide()
			end
		end
	end
end